@@ -686,7 +686,7 @@ class BalloonWindow(object):
     def show_sstp_message(self, message, sender):
         if self.sstp_region is None:
             self.show()
-        self.sstp_message = '{0} ({1})'.format(message, sender)
+        self.sstp_message = u'{0} ({1})'.format(message, sender)
         x, y, w, h = self.sstp_region
         self.sstp_layout.set_text(self.sstp_message)
         message_width, message_height = self.sstp_layout.get_pixel_size()
@@ -695,7 +695,7 @@ class BalloonWindow(object):
             i = 0
             while 1:
                 i += 1
-                s = '{0}... ({1})'.format(message[:i], sender)
+                s = u'{0}... ({1})'.format(message[:i], sender)
                 self.sstp_layout.set_text(s)
                 message_width, message_height = \
                                self.sstp_layout.get_pixel_size()
@@ -1374,7 +1374,7 @@ class TeachBox(CommunicateWindow):
     def send(self, data):
         if data:
             data = unicode(data, 'utf-8')
-        self.request_parent('NOTIFT', 'notify_user_teach', data)
+        self.request_parent('NOTIFY', 'notify_user_teach', data)
         self.request_parent('NOTIFY', 'reset_user_interaction')
 
 
