@@ -35,17 +35,24 @@ class Config(dict):
         return default
 
     def __str__(self):
-        return ''.join(['{0},{1}\n'.format(key, value) for key, value in self.items()]) ## FIXME
+        return ''.join(
+            ['{0},{1}\n'.format(key.encode('utf-8'),
+                                value.encode('utf-8')) \
+                 for key, value in self.items()])
 
 
 def create_from_file(path):
-    with open(path) as f:
-        buf = [line.strip() for line in f if line.strip()]
-    return create_from_buffer(buf)
-
-def create_from_buffer(buf):
-    dic = Config()
     charset = 'Shift_JIS' # default
+    with open(path) as f:
+        if f.read(3) == codecs.BOM_UTF8:
+            charset = 'UTF-8'
+        else:
+            f.seek(0) # rewind
+        buf = [line.strip() for line in f if line.strip()]
+    return create_from_buffer(buf, charset)
+
+def create_from_buffer(buf, charset='Shift_JIS'):
+    dic = Config()
     for line in buf:
         try:
             key, value = line.split(',', 1)
