@@ -142,11 +142,13 @@ class Saori(SAORI):
         elif name == 'clear':
             for name in self.blns:
                 data, bln = self.blns[name]
-                if bln_id in bln and \
-                   bln[bln_id].get_state() != 'orusuban':
+                new_bln = {}
+                for bln_id in bln:
+                    if bln[bln_id].get_state() == 'orusuban':
+                        new_bln[bln_id] = bln[bln_id]
+                        continue
                     bln[bln_id].destroy()
-                    del bln[bln_id]
-                    self.blns[name] = (data, {})
+                self.blns[name] = (data, new_bln)
         return None
 
 
@@ -242,6 +244,8 @@ class Balloon(object):
                 fontcolor_g = int(data['font.color'][2:4], 16) /255.0
                 fontcolor_b = int(data['font.color'][4:6], 16) / 255.0
                 self.fontcolor = (fontcolor_r, fontcolor_g, fontcolor_b)
+            else:
+                self.fontcolor = (0.0, 0.0, 0.0) # XXX
             default_font_size = 12 # for Windows environment
             self.font_size = int(data.get('font.size', default_font_size))
             self.layout = pango.Layout(self.darea.get_pango_context())
@@ -306,7 +310,7 @@ class Balloon(object):
             self.processed_script = None
             logging.error('-' * 50)
             logging.error(e)
-            logging.error(self.script)
+            logging.error(self.script.encode('utf-8'))
         self.timeout_id = glib.timeout_add(10, self.do_idle_tasks)
 
     def set_position(self):
@@ -435,7 +439,7 @@ class Balloon(object):
             self.processed_script = None
             logging.error('-' * 50)
             logging.error(e)
-            logging.error(self.script)
+            logging.error(self.script.encode('utf-8'))
 
     def get_sakura_is_talking(self):
         talking = 0
