@@ -12,6 +12,7 @@
 #  PURPOSE.  See the GNU General Public License for more details.
 #
 
+import codecs
 import logging
 import os
 import re
@@ -457,11 +458,18 @@ def read_surfaces_txt(surface_dir):
             buf = []
             key = None
             opened = False
+            if f.read(3) == codecs.BOM_UTF8:
+                charset = 'UTF-8'
+            else:
+                f.seek(0) # rewind
             for line in f:
                 if line.startswith('#') or line.startswith('//'):
                     continue
-                # '\x81\x40': full-width space in Shift_JIS
-                temp = line.replace('\x81\x40', '').strip()
+                if charset == 'Shift_JIS':
+                    # '\x81\x40': full-width space in Shift_JIS
+                    temp = line.replace('\x81\x40', '').strip()
+                else:
+                    temp = line.strip()
                 if not temp:
                     continue
                 if temp.startswith('charset'):
@@ -594,6 +602,10 @@ def read_plugin_txt(src_dir):
             menu_items = []
             error = None
             lineno = 0
+            if f.read(3) == codecs.BOM_UTF8:
+                charset = 'UTF-8' # XXX
+            else:
+                f.seek(0) # rewind
             for line in f:
                 lineno += 1
                 if not line.strip() or line.startswith('#'):
@@ -662,7 +674,7 @@ def test():
         desc, shiori_dir, use_makoto, surface_set, prefix, shiori_dll, shiori_name = ghosts[key]
         print 'GHOST', '=' * 50
         print prefix
-        print str(desc).encode('utf-8', 'ignore')
+        print str(desc)
         print shiori_dir
         print shiori_dll
         print shiori_name
@@ -671,10 +683,10 @@ def test():
             for name, surface_dir, desc, alias, surface, tooltips in surface_set.values():
                 print '-' * 50
                 print 'surface:', name.encode('utf-8', 'ignore')
-                print str(desc).encode('utf-8', 'ignore')
+                print str(desc)
                 for k, v in surface.items():
                     print k, '=', v[0]
-                    print str(v[1]).encode('utf-8', 'ignore')
+                    print str(v[1])
                 if alias:
                     buf = []
                     for k, v in alias.items():
@@ -695,10 +707,10 @@ def test():
     for key in balloons:
         desc, balloon = balloons[key]
         print 'BALLOON', '=' * 50
-        print str(desc).encode('utf-8', 'ignore')
+        print str(desc)
         for k, v in balloon.items():
             print k, '=', v[0]
-            print str(v[1]).encode('utf-8', 'ignore')
+            print str(v[1])
     # plugins
     for plugin_name, plugin_dir, startup, menu_items in plugins:
         print 'PLUGIN', '=' * 50
