@@ -104,11 +104,13 @@ class Installer(object):
             with zipfile.ZipFile(filename) as zf:
                 for name in zf.namelist():
                     uname = name.decode('cp932') # XXX
-                    buf = zf.read(name)
                     path = os.path.join(tmpdir, uname.encode(self.fs_encoding))
                     dname, fname = os.path.split(path)
                     if not os.path.exists(dname):
                         os.makedirs(dname)
+                    if not fname: # directory
+                        continue
+                    buf = zf.read(name)
                     with open(path, 'wb') as of:
                         of.write(buf)
         except:
