@@ -68,83 +68,84 @@ class Menu(object):
           </popup>
         </ui>
         '''
-        self.__menu_list = { ## FIXME
-            'Portal': [
-                ('Portal', None, _('Portal sites(_P)'), None),
-                None, 1, 1],
-            'Recommend': [
-                ('Recommend', None, _('Recommend sites(_R)'), None),
-                None, 1, 1],
-            'Options': [
-                ('Options', None, _('Options(_F)'), None),
-                None, 1, 1],
-            'Options/Update': [
-                ('Update', None, _('Network Update(_U)'), None,
-                 '', lambda *a: self.request_parent('NOTIFY', 'network_update')),
-                None, 1, 1],
-            'Options/Vanish': [
-                ('Vanish', None, _('Vanish(_F)'), None,
-                 '', lambda *a: self.request_parent('NOTIFY', 'vanish_sakura')),
-                None, 1, 1],
-            'Options/Preferences': [
-                ('Preferences', None, _('Preferences...(_O)'), None,
-                 '', lambda *a: self.request_parent('NOTIFY', 'edit_preferences')),
-                None, 1, 1],
-            'Options/Console': [
-                ('Console', None, _('Console(_C)'), None,
-                 '', lambda *a: self.request_parent('NOTIFY', 'open_console')),
-                None, 1, 1],
-            'Options/Manager': [
-                ('Manager', None, _('Ghost Manager(_M)'), None,
-                 '', lambda *a: self.request_parent('NOTIFY', 'open_ghost_manager')),
-                None, 1, 1],
-            'Information': [
-                ('Information', None, _('Information(_I)'), None),
-                None, 1, 1],
-            'Information/Usage': [
-                ('Usage', None, _('Usage graph(_A)'), None,
-                 '', lambda *a: self.request_parent('NOTIFY', 'show_usage')),
-                None, 1, 1],
-            'Information/Version': [
-                ('Version', None, _('Version(_V)'), None,
-                 '', lambda *a: self.request_parent('NOTIFY', 'about')),
-                None, 1, 1],
-            'Close': [
-                ('Close', None, _('Close(_W)'), None,
-                 '', lambda *a: self.request_parent('NOTIFY', 'close_sakura')),
-                None, 1, 1],
-            'Quit': [('Quit', None, _('Quit(_Q)'), None,
-                      '', lambda *a: self.request_parent('NOTIFY', 'close_all')),
-                     None, 1, 1],
-            'Change': [
-                ('Change', None, _('Change(_G)'), None),
-                None, 1, 1],
-            'Summon': [
-                ('Summon', None, _('Summon(_X)'), None),
-                None, 1, 1],
-            'Shell': [
-                ('Shell', None, _('Shell(_S)'), None),
-                None, 1, 1],
-            'Balloon': [
-                ('Balloon', None, _('Balloon(_B)'), None),
-                None, 1, 1],
-            'Costume': [
-                ('Costume', None, _('Costume(_C)'), None),
-                None, 1, 1],
-            'Stick': [
-                ('Stick', None, _('Stick(_Y)'), None,
-                 '', lambda *a: self.request_parent('NOTIFY', 'stick_window'),
-                 False),
-                None, 1, 1],
-            'Nekodorif': [
-                ('Nekodorif', None, _('Nekodorif(_N)'), None),
-                None, 1, 1],
-            'Kinoko': [
-                ('Kinoko', None, _('Kinoko(_K)'), None),
-                None, 1, 1],
-            'Plugin': [
-                ('Plugin', None, _('Plugin(_P)'), None),
-                None, 1, 1],
+        self.__menu_list = {
+            'Portal': {
+                'entry': ('Portal', None, _('Portal sites(_P)'), None),
+                'visible': 1},
+            'Recommend': {
+                'entry': ('Recommend', None, _('Recommend sites(_R)'), None),
+                'visible': 1},
+            'Options': {
+                'entry': ('Options', None, _('Options(_F)'), None),
+                'visible': 1},
+            'Options/Update': {
+                'entry': ('Update', None, _('Network Update(_U)'), None,
+                          '', lambda *a: self.request_parent('NOTIFY', 'network_update')),
+                'visible': 1},
+            'Options/Vanish': {
+                'entry': ('Vanish', None, _('Vanish(_F)'), None,
+                          '', lambda *a: self.request_parent('NOTIFY', 'vanish')),
+                'visible': 1},
+            'Options/Preferences': {
+                'entry': ('Preferences', None, _('Preferences...(_O)'), None,
+                           '', lambda *a: self.request_parent('NOTIFY', 'edit_preferences')),
+                'visible': 1},
+            'Options/Console': {
+                'entry': ('Console', None, _('Console(_C)'), None,
+                          '', lambda *a: self.request_parent('NOTIFY', 'open_console')),
+                'visible': 1},
+            'Options/Manager': {
+                'entry': ('Manager', None, _('Ghost Manager(_M)'), None,
+                          '', lambda *a: self.request_parent('NOTIFY', 'open_ghost_manager')),
+                'visible': 1},
+            'Information': {
+                'entry': ('Information', None, _('Information(_I)'), None),
+                'visible': 1},
+            'Information/Usage': {
+                'entry': ('Usage', None, _('Usage graph(_A)'), None,
+                          '', lambda *a: self.request_parent('NOTIFY', 'show_usage')),
+                'visible': 1},
+            'Information/Version': {
+                'entry': ('Version', None, _('Version(_V)'), None,
+                          '', lambda *a: self.request_parent('NOTIFY', 'about')),
+                'visible': 1},
+            'Close': {
+                'entry': ('Close', None, _('Close(_W)'), None,
+                          '', lambda *a: self.request_parent('NOTIFY', 'close_sakura')),
+                'visible': 1},
+            'Quit': {
+                'entry': ('Quit', None, _('Quit(_Q)'), None,
+                          '', lambda *a: self.request_parent('NOTIFY', 'close_all')),
+                'visible': 1},
+            'Change': {
+                'entry': ('Change', None, _('Change(_G)'), None),
+                'visible': 1},
+            'Summon': {
+                'entry': ('Summon', None, _('Summon(_X)'), None),
+                'visible': 1},
+            'Shell': {
+                'entry': ('Shell', None, _('Shell(_S)'), None),
+                'visible': 1},
+            'Balloon': {
+                'entry': ('Balloon', None, _('Balloon(_B)'), None),
+                'visible': 1},
+            'Costume': {
+                'entry': ('Costume', None, _('Costume(_C)'), None),
+                'visible': 1},
+            'Stick': {
+                'entry': ('Stick', None, _('Stick(_Y)'), None,
+                          '', lambda *a: self.request_parent('NOTIFY', 'stick_window'),
+                          False),
+                'visible': 1},
+            'Nekodorif': {
+                'entry': ('Nekodorif', None, _('Nekodorif(_N)'), None),
+                'visible': 1},
+            'Kinoko': {
+                'entry': ('Kinoko', None, _('Kinoko(_K)'), None),
+                'visible': 1},
+            'Plugin': {
+                'entry': ('Plugin', None, _('Plugin(_P)'), None),
+                'visible': 1},
             }
         self.__pixmap = None
         self.__pixmap_with_sidebar = None
@@ -153,10 +154,10 @@ class Menu(object):
         self.__fontcolor = {'normal': '#000000', 'prelight': '#ffffff'} ## FIXME
         self.__sidebar_width = 0
         actions = gtk.ActionGroup('Actions')
-        entry = [value[0] for key, value in self.__menu_list.items() \
+        entry = [value['entry'] for key, value in self.__menu_list.items() \
                      if key != 'Stick']
         actions.add_actions(tuple(entry))
-        actions.add_toggle_actions(tuple([self.__menu_list['Stick'][0]]))
+        actions.add_toggle_actions(tuple([self.__menu_list['Stick']['entry']]))
         self.ui_manager = gtk.UIManager()
         self.ui_manager.insert_action_group(actions, 0)
         self.ui_manager.add_ui_from_string(ui_info)
@@ -182,6 +183,7 @@ class Menu(object):
             self.__pixmap_with_sidebar = None
             self.__pixmap_prelight = None
             self.__pixmap_prelight_with_sidebar = None
+            self.__sidebar_width = 0
             return
         pixmap, mask = pixbuf.render_pixmap_and_mask(255)
         self.__pixmap = pixmap
@@ -322,8 +324,7 @@ class Menu(object):
 
     def __update_ui(self, side):
         for key in self.__ui:
-            if key not in self.__menu_list:
-                continue
+            assert key in self.__menu_list
             if side > 1:
                 if key in ['Options/Update', 'Options/Vanish']:
                     name_list = self.__ui[key][0][1] # same as 'kero'
@@ -363,10 +364,9 @@ class Menu(object):
         item_list = submenu.get_children()
         if item_list:
             for item in item_list:
-                if self.__pixmap_prelight:
-                    style = item.style.copy()
-                    style.bg_pixmap[gtk.STATE_PRELIGHT] = self.__pixmap_prelight
-                    item.set_style(style)
+                style = item.style.copy()
+                style.bg_pixmap[gtk.STATE_PRELIGHT] = self.__pixmap_prelight
+                item.set_style(style)
                 submenu = item.get_submenu()
                 if submenu:
                     style = submenu.style.copy()
@@ -422,22 +422,18 @@ class Menu(object):
         self.__set_mayuna_menu(side)
         self.__set_nekodorif_menu()
         self.__set_kinoko_menu()
-        if self.__pixmap_with_sidebar:
-            style = self.__popup_menu.style.copy()
-            style.bg_pixmap[gtk.STATE_NORMAL] = self.__pixmap_with_sidebar
-            self.__popup_menu.set_style(style)
+        style = self.__popup_menu.style.copy()
+        style.bg_pixmap[gtk.STATE_NORMAL] = self.__pixmap_with_sidebar
+        self.__popup_menu.set_style(style)
         for key in self.__menu_list:
             item = self.ui_manager.get_widget(''.join(('/popup/', key)))
-            visible = self.__menu_list[key][2]
-            changed = self.__menu_list[key][3]
-            if item and changed:
-                self.__menu_list[key][3] = 0 # changed
+            visible = self.__menu_list[key]['visible']
+            if item:
                 if visible:
                     if not key.startswith('Options/'): # XXX
-                        if self.__pixmap_prelight_with_sidebar:
-                            style = item.style.copy()
-                            style.bg_pixmap[gtk.STATE_PRELIGHT] = self.__pixmap_prelight_with_sidebar
-                            item.set_style(style)
+                        style = item.style.copy()
+                        style.bg_pixmap[gtk.STATE_PRELIGHT] = self.__pixmap_prelight_with_sidebar
+                        item.set_style(style)
                     label = item.get_children()[0]
                     style = label.style.copy()
                     cmap = label.get_colormap()
@@ -449,11 +445,10 @@ class Menu(object):
                     item.show()
                     submenu = item.get_submenu()
                     if submenu:
-                        if self.__pixmap:
-                            style = submenu.style.copy()
-                            style.bg_pixmap[gtk.STATE_NORMAL] = self.__pixmap
-                            submenu.set_style(style)
-                            self.set_submenu_back_pixmap(submenu)
+                        style = submenu.style.copy()
+                        style.bg_pixmap[gtk.STATE_NORMAL] = self.__pixmap
+                        submenu.set_style(style)
+                        self.set_submenu_back_pixmap(submenu)
                         self.set_submenu_fontcolor(submenu)
                 else:
                     item.hide()
@@ -465,7 +460,7 @@ class Menu(object):
                 allocation.x += self.__sidebar_width
                 allocation.width -= self.__sidebar_width
                 item.size_allocate(allocation)
-        self.__popup_menu.resize_children()
+        self.__popup_menu.resize_children() ## FIXME: force resize popup_menu
 
     def __set_caption(self, name, caption):
         assert name in self.__menu_list
@@ -474,15 +469,11 @@ class Menu(object):
         if item:
             label = item.get_children()[0]
             label.set_text_with_mnemonic(caption)
-        self.__menu_list[name][3] = 1 # changed
 
     def __set_visible(self, name, visible):
         assert name in self.__menu_list
         assert visible in [0, 1]
-        if self.__menu_list[name][2] == visible:
-            return
-        self.__menu_list[name][2] = visible
-        self.__menu_list[name][3] = 1 # changed
+        self.__menu_list[name]['visible'] = visible
 
     def __set_portal_menu(self, side, portal):
         if side >= 1:
@@ -522,7 +513,6 @@ class Menu(object):
                 self.__set_visible('Portal', 1)
             else:
                 self.__set_visible('Portal', 0)
-        self.__menu_list['Portal'][3] = 1 # changed
 
     def __set_recommend_menu(self, recommend):
         if recommend:
@@ -559,103 +549,67 @@ class Menu(object):
             self.__set_visible('Recommend', 1)
         else:
             self.__set_visible('Recommend', 0)
-        self.__menu_list['Recommend'][3] = 1 # changed
 
-    def __update_ghost_menu(self, set_list, summon):
-        if summon:
-            menu_type = 'start_sakura'
-        else:
-            menu_type = 'select_sakura'
-        ghost_menu = gtk.Menu()
-        for i in range(len(set_list)):
-            if set_list[i] is None:
-                continue
-            name = set_list[i]['name']
-            icon = set_list[i]['icon']
-            shell_list = set_list[i]['shell']
-            if icon is not None:
+    def create_ghost_menuitem(self, name, icon, key, handler):
+        if icon is not None:
+            pixbuf = ninix.pix.create_icon_pixbuf(icon)
+            if pixbuf is None:
+                item = gtk.MenuItem(name)
+            else:
                 image = gtk.Image()
-                image.set_from_file(icon)
+                image.set_from_pixbuf(pixbuf)
                 image.show()
                 item = gtk.ImageMenuItem(name)
                 item.set_image(image)
-            else:
-                item = gtk.MenuItem(name)
-            item.set_name('popup menu item')
-            item.show()
-            ghost_menu.append(item)
-            if set_list[i]['instance'].is_running():
-                item.set_sensitive(False)
-            if len(shell_list) <= 1:
-                shell_name, value = shell_list[0]
-                item.connect(
-                    'activate',
-                    lambda a, v: self.request_parent('NOTIFY', menu_type, v),
-                    (value))
-            else:
-                submenu = gtk.Menu()
-                submenu.set_name('popup menu')
-                item.set_submenu(submenu)
-                for shell_name, value in shell_list:
-                    item = gtk.MenuItem(shell_name)
-                    item.set_name('popup menu item')
-                    item.connect(
-                        'activate',
-                        lambda a, v: self.request_parent(
-                            'NOTIFY', menu_type, v),
-                        (value))
-                    item.show()
-                    submenu.append(item)
-        path = 'Summon' if summon else 'Change'
-        menuitem = self.ui_manager.get_widget(''.join(('/popup/', path)))
-        menuitem.set_submenu(ghost_menu)
-        self.__menu_list[path][3] = 1
+                item.set_always_show_image(True) # XXX
+        else:
+            item = gtk.MenuItem(name)
+        item.set_name('popup menu item')
+        item.show()
+        item.connect('activate', lambda a, v: handler(v), (key))
+        return item
 
     def __set_ghost_menu(self):
-        ghosts = self.request_parent('GET', 'get_ghost_list')
-        self.__update_ghost_menu(ghosts, False)
-        self.__update_ghost_menu(ghosts, True)
+        for path in ['Summon', 'Change']:
+            ghost_menu = gtk.Menu()
+            for items in self.request_parent('GET', 'get_ghost_menus'):
+                item = items[path]
+                if item.get_parent():
+                    item.reparent(ghost_menu)
+                else:
+                    ghost_menu.append(item)
+            menuitem = self.ui_manager.get_widget(''.join(('/popup/', path)))
+            menuitem.set_submenu(ghost_menu)
 
     def __set_shell_menu(self):
-        shell_list = self.request_parent('GET', 'get_shell_list')
-        shell_menu = gtk.Menu()
-        for i in range(len(shell_list)):
-            shell_name, value, working = shell_list[i]
-            item = gtk.MenuItem(shell_name)
-            item.set_name('popup menu item')
-            item.connect(
-                'activate',
-                lambda a, v: self.request_parent('NOTIFY', 'select_shell', v),
-                (value))
-            item.show()
-            if working:
-                item.set_sensitive(False)
-            shell_menu.append(item)
+        shell_menu = self.request_parent('GET', 'get_shell_menu')
         menuitem = self.ui_manager.get_widget(''.join(('/popup/', 'Shell')))
+        if menuitem.get_submenu():
+            menuitem.remove_submenu()
         menuitem.set_submenu(shell_menu)
-        self.__menu_list['Shell'][3] = 1
 
     def __set_balloon_menu(self):
-        balloon_list = self.request_parent('GET', 'get_balloon_list')
-        balloon_menu = gtk.Menu()
-        group = None
-        for name, directory in balloon_list:
-            item = group = gtk.RadioMenuItem(group, name)
-            item.set_name('popup menu item')
-            hid = item.connect(
-                'activate',
-                lambda w, d: self.request_parent(
-                    'NOTIFY', 'select_balloon', w, d),
-                directory)
-            item.show()
-            balloon_menu.append(item)
-            if directory == self.request_parent('GET', 'get_current_balloon_directory'):
-                item.handler_block(hid)
-                item.activate()
-                item.handler_unblock(hid)
+        balloon_menu = self.request_parent('GET', 'get_balloon_menu')
         menuitem = self.ui_manager.get_widget(''.join(('/popup/', 'Balloon')))
+        if menuitem.get_submenu():
+            menuitem.remove_submenu()
         menuitem.set_submenu(balloon_menu)
-        self.__menu_list['Balloon'][3] = 1
+
+    def create_meme_menu(self, menuitem):
+        menu = gtk.Menu()
+        for item in menuitem.values():
+            if item.get_parent():
+                item.reparent(menu)
+            else:
+                menu.append(item)
+        return menu
+
+    def create_meme_menuitem(self, name, value, handler):
+        item = gtk.MenuItem(name)
+        item.set_name('popup menu item')
+        item.show()
+        item.connect('activate', lambda a, v: handler(v), (value))
+        return item
 
     def __set_plugin_menu(self):
         plugin_list = self.request_parent('GET', 'get_plugin_list')
@@ -686,7 +640,7 @@ class Menu(object):
                     item.connect(
                         'activate',
                         lambda a, v: self.request_parent(
-                            'NOTIFY',  'select_plugin', v),
+                            'NOTIFY', 'select_plugin', v),
                         (value))
                     item.show()
                     ##if working:
@@ -694,7 +648,6 @@ class Menu(object):
                     submenu.append(item)
         menuitem = self.ui_manager.get_widget(''.join(('/popup/', 'Plugin')))
         menuitem.set_submenu(plugin_menu)
-        self.__menu_list['Plugin'][3] = 1
 
     def __set_nekodorif_menu(self):
         nekodorif_list = self.request_parent('GET', 'get_nekodorif_list')
@@ -715,7 +668,6 @@ class Menu(object):
         menuitem = self.ui_manager.get_widget(
             ''.join(('/popup/', 'Nekodorif')))
         menuitem.set_submenu(nekodorif_menu)
-        self.__menu_list['Nekodorif'][3] = 1
 
     def __set_kinoko_menu(self):
         kinoko_list = self.request_parent('GET', 'get_kinoko_list')
@@ -735,7 +687,6 @@ class Menu(object):
             ##    item.set_sensitive(False)
         menuitem = self.ui_manager.get_widget(''.join(('/popup/', 'Kinoko')))
         menuitem.set_submenu(kinoko_menu)
-        self.__menu_list['Kinoko'][3] = 1
 
     def get_stick(self):
         item =  self.ui_manager.get_widget(''.join(('/popup/', 'Stick')))
