@@ -208,6 +208,20 @@ def get_png_IHDR(path):
         buf = f.read(24)
     return buf
 
+def __pixbuf_new_from_file(path):
+    if os.name == 'nt': # XXX
+        path = unicode(path, 'mbcs').encode('utf-8')
+    return gtk.gdk.pixbuf_new_from_file(path)
+
+def create_icon_pixbuf(path):
+    try:
+        pixbuf = __pixbuf_new_from_file(path)
+    except: # compressed icons are not supported. :-(
+        pixbuf = None
+    else:
+        pixbuf = pixbuf.scale_simple(16, 16, gtk.gdk.INTERP_BILINEAR)
+    return pixbuf
+
 def create_blank_pixbuf(width, height):
     pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, width, height)
     pixbuf.fill(0xffffffffL)
@@ -232,7 +246,7 @@ def create_pixbuf_from_DGP_file(path):
     key_length = len(key)
     if key_length == 0: # not encrypted
         logging.warning(''.join((filename, ' generates a null key.')))
-        pixbuf = gtk.gdk.pixbuf_new_from_file(filename)
+        pixbuf = __pixbuf_new_from_file(filename)
         return pixbuf
     key = ''.join((key[1:], key[0]))
     key_pos = 0
@@ -271,7 +285,7 @@ def create_pixbuf_from_file(path, is_pnr=True, use_pna=False):
     elif ext == '.ddp':
         pixbuf = create_pixbuf_from_DDP_file(path)
     else:
-        pixbuf = gtk.gdk.pixbuf_new_from_file(path)
+        pixbuf = __pixbuf_new_from_file(path)
     if is_pnr:
         array = pixbuf.get_pixels_array()
         if not pixbuf.get_has_alpha():
@@ -289,7 +303,7 @@ def create_pixbuf_from_file(path, is_pnr=True, use_pna=False):
         path = os.path.join(head, ''.join((basename, '.pna')))
         if os.path.exists(path):
             assert pixbuf.get_has_alpha()
-            pna_pixbuf = gtk.gdk.pixbuf_new_from_file(path)
+            pna_pixbuf = __pixbuf_new_from_file(path)
             pna_array = pna_pixbuf.get_pixels_array()
             assert pna_pixbuf.get_bits_per_sample() / 8 == 1
             pixbuf_array = pixbuf.get_pixels_array()
