@@ -84,7 +84,7 @@ class BasePlugin(multiprocessing.Process):
                     message = ''.join((message, 'IfGhost: ', if_ghost, '\r\n'))
                 message = ''.join((message, 'Script: ', script, '\r\n'))
         message = ''.join((message, '\r\n'))
-        sstp.send(message)
+        sstp.send(message.encode('utf-8'))
         buffer = []
         while 1:
             buffer.append(sstp.recv(1024))
@@ -102,7 +102,7 @@ class BasePlugin(multiprocessing.Process):
             line = line.strip('\r\n\r\n')
             if not line:
                 continue
-            return line
+            return unicode(line, 'utf-8')
         return ''
 
     def notify_sstp(self, sender, event, ref=[], entry=[], script_odict=None):
@@ -124,7 +124,7 @@ class BasePlugin(multiprocessing.Process):
                     message = ''.join((message, 'IfGhost: ', if_ghost, '\r\n'))
                 message = ''.join((message, 'Script: ', script, '\r\n'))
         message = ''.join((message, '\r\n'))
-        sstp.send(message)
+        sstp.send(message.encode('utf-8'))
         buffer = []
         while 1:
             buffer.append(sstp.recv(1024))
@@ -142,7 +142,7 @@ class BasePlugin(multiprocessing.Process):
             line = line.strip('\r\n\r\n')
             if not line:
                 continue
-            return line
+            return unicode(line, 'utf-8')
         return ''
 
     def send_script(self, sender, script, if_ghost=None):
