@@ -212,7 +212,7 @@ class Sakura(object):
         if char > 2:
             self.char = char
         # XXX
-        if self.desc.get('name') == 'BTH小っちゃいってことは便利だねっ':
+        if self.desc.get('name') == u'BTH小っちゃいってことは便利だねっ':
             self.set_SSP_mode(1)
         else:
             self.set_SSP_mode(0)
@@ -305,7 +305,8 @@ class Sakura(object):
                 self.shiori.show_description()
         else:
             logging.error('{0} cannot load SHIORI({1})'.format(
-                    self.get_selfname(), self.shiori_name))
+                    self.get_selfname().encode('utf-8', 'ignore'),
+                    self.shiori_name))
         self.__charset = 'Shift_JIS' # default
         self.get_event_response('basewareversion',
                                 ninix.version.VERSION,
@@ -494,11 +495,11 @@ class Sakura(object):
             (self))
 
     def get_ifghost(self):
-        return '{0},{1}'.format(self.get_selfname(), self.get_keroname())
+        return ''.join((self.get_selfname(), ',', self.get_keroname()))
 
     def ifghost(self, ifghost):
         names = self.get_ifghost()
-        name = '{0}'.format(self.get_selfname())
+        name = self.get_selfname()
         return bool(ifghost in [name, names])
 
     def get_name(self, default=unicode(_('Sakura&Unyuu'), 'utf-8')):
@@ -535,7 +536,7 @@ class Sakura(object):
         return self.translate(result)
 
     def getword(self, word_type):
-        result = self.get_event_response('{0}'.format(word_type))
+        result = self.get_event_response(word_type)
         return self.translate(result)
 
     def getstring(self, name):
@@ -596,10 +597,13 @@ class Sakura(object):
                           'SecurityLevel: local\r\n',
                           'Charset: {0}\r\n'.format(self.__charset)))
         for i in range(len(ref)):
-            if ref[i] is not None:
+            value = ref[i]
+            if value is not None:
+                value = value if isinstance(value, basestring) \
+                    else str(value)
                 header = ''.join((header,
                                   'Reference', str(i), ': ',
-                                  str(ref[i]), '\r\n'))
+                                  value, '\r\n'))
         header = ''.join((header, '\r\n'))
         header = header.encode(self.__charset, 'ignore')
         response = self.shiori.request(header)
@@ -825,9 +829,11 @@ class Sakura(object):
             for n in range(len(arglist)):
                 value = arglist[n]
                 if value is not None:
+                    value = value if isinstance(value, basestring) \
+                        else str(value)
                     logging.debug(
                     'Reference{0:d}: {1}'.format(
-                        n, str(value).encode('utf-8', 'ignore')))
+                        n, value.encode('utf-8', 'ignore')))
         if not script: # an empty script is ignored
             if event in self.boot_event:
                 self.surface_bootup()
@@ -928,7 +934,8 @@ class Sakura(object):
         self.balloon.set_balloon_default()
         self.position_balloons()
         name = desc.get('name', '')
-        logging.info('balloon {0} {1}'.format(name, path))
+        logging.info('balloon {0} {1}'.format(
+                name.encode('utf-8', 'ignore'), path))
         self.notify_event('OnBalloonChange', name, path)
 
     def surface_bootup(self):
@@ -1092,13 +1099,12 @@ class Sakura(object):
                     'GET', 'find_balloon_by_name', balloon_name)
         if balloon is None:
             if self.balloon_directory is not None:
-                default_balloon = self.balloon_directory
+                balloon = self.balloon_directory
             else:
-                default_balloon = self.request_parent(
+                balloon = self.request_parent(
                     'GET', 'get_preference', 'default_balloon')
-            balloon = self.request_parent(
-                'GET', 'get_balloon_description', default_balloon)
-        desc, balloon = balloon
+        desc, balloon = self.request_parent(
+                'GET', 'get_balloon_description', balloon)
         self.set_balloon(desc, balloon)
         if not temp:
             self.load_shiori()
@@ -1301,7 +1307,7 @@ class Sakura(object):
                 self.processed_script.extend(self.script_parser.parse(script))
             except ninix.script.ParserError as e:
                 logging.error('-' * 50)
-                logging.error(('{0}'.format(e)).encode('utf-8', 'ignore'))
+                logging.error('{0}'.format(e)) # 'UTF-8'
                 done, script = e
                 self.processed_script.extend(done)
             else:
