@@ -87,7 +87,7 @@ class ParserError(Exception):
         else:
             column = '??'
             dump = self.src
-        return 'ParserError: column {0}: {1}\n{2}'.format(column, self.message, dump)
+        return 'ParserError: column {0}: {1}\n{2}'.format(column, self.message, dump.encode('utf-8'))
 
 
 class Parser(object):
@@ -444,7 +444,7 @@ def test_tokenizer():
     parser = Parser()
     for test in testcases:
         try:
-            print parser.tokenize(test)
+            print parser.tokenize(unicode(test, 'utf-8'))
         except ParserError as e:
             print e
 
@@ -454,6 +454,7 @@ def test_parser(error='strict'):
         print '*' * 60
         print test
         script = []
+        test = unicode(test, 'utf-8')
         while 1:
             try:
                 script.extend(parser.parse(test))
@@ -473,8 +474,8 @@ def print_script_tree(tree):
             name, args = node[1], node[2:]
             print 'TAG', name
             for n in range(len(args)):
-                if isinstance(args[n], str):
-                    print '\tARG#{0:d}\t{1}'.format(n + 1, args[n])
+                if isinstance(args[n], basestring):
+                    print '\tARG#{0:d}\t{1}'.format(n + 1, args[n].encode('utf-8'))
                 else:
                     print '\tARG#{0:d}\tTEXT'.format(n + 1)
                     print_text(args[n], 2)
@@ -485,12 +486,12 @@ def print_script_tree(tree):
 def print_text(text, indent):
     for chunk in text:
         if chunk[0] == TEXT_STRING:
-            print ''.join(('\t' * indent, 'STRING\t"{0}"'.format(chunk[1])))
+            print ''.join(('\t' * indent, 'STRING\t"{0}"'.format(chunk[1].encode('utf-8'))))
         elif chunk[0] == TEXT_META:
             name, args = chunk[1], chunk[2:]
             print ''.join(('\t' * indent, 'META\t', name))
             for n in range(len(args)):
-                print ''.join(('\t' * indent, '\tARG#{0:d}\t{1}'.format(n + 1, args[n])))
+                print ''.join(('\t' * indent, '\tARG#{0:d}\t{1}'.format(n + 1, args[n].encode('utf-8'))))
 
 
 if __name__ == '__main__':
