@@ -313,7 +313,8 @@ class PluginControler:
             for name, value in self.data[plugin_dir].items():
                 if value is None:
                     continue
-                f.write('{0}:{1}\n'.format(name, value))
+                f.write('{0}:{1}\n'.format(name.encode('utf-8'),
+                                           value.encode('utf-8')))
 
     def load_data(self, plugin_dir):
         home_dir = ninix.home.get_ninix_home()
@@ -337,6 +338,7 @@ class PluginControler:
                     return {}
                 for line in f:
                     line = line.rstrip('\r\n')
+                    line = unicode(line, 'utf-8')
                     key, value = line.split(':', 1)
                     data[key] = value
         except IOError as e:
@@ -1084,7 +1086,7 @@ class Application(object):
         history = {}
         for key in self.ghosts:
             sakura = self.ghosts[key].instance
-            name = sakura.get_name(default=i)
+            name = sakura.get_name(default=key)
             ghost_time = 0
             prefix = sakura.get_prefix()
             path = os.path.join(prefix, 'HISTORY')
